//+------------------------------------------------------------------+
//|                                                HOPS and LOPS.mq4 |
//|                      Copyright  2006, MetaQuotes Software Corp. |
//|                                        http://www.metaquotes.net |
//+------------------------------------------------------------------+
#property copyright "Copyright  2006, MetaQuotes Software Corp."
#property link      "http://www.metaquotes.net"

#property indicator_chart_window
#property indicator_buffers 2
#property indicator_color1 Blue
#property indicator_color2 Red

//---- input parameters
extern int       ShiftTime = 1;
int       ExtFormula= 0;
extern color     HOPS= Red;
extern color     LOPS= Blue;
//---- buffers
double ExtMapBuffer1[];
double ExtMapBuffer2[];

//+------------------------------------------------------------------+
//| Deleting lines function                                          |
//+------------------------------------------------------------------+
void DeleteLines()
  {
   ObjectDelete("Pivot_Line");
   ObjectDelete("LOPS2_Line");
   ObjectDelete("LOPS1_Line");
   ObjectDelete("HOPS2_Line");
   ObjectDelete("HOPS1_Line");
   ObjectDelete("LOPS2 label");
   ObjectDelete("LOPS1 label");
   ObjectDelete("HOPS2 label");
   ObjectDelete("HOPS1 label");
   ObjectDelete("Pivot label");

   ObjectsRedraw();
  }
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
   if(MathAbs(ShiftTime)>23) ShiftTime=0;
   Comment("Shift Hours = ", ShiftTime);
   if(ShiftTime<0) ShiftTime+=24;
   ShiftTime*=3600;
//---- indicators
   SetIndexStyle(0,DRAW_NONE);
   SetIndexBuffer(0,ExtMapBuffer1);
   SetIndexEmptyValue(0,0.0);
   SetIndexStyle(1,DRAW_NONE);
   SetIndexBuffer(1,ExtMapBuffer2);
   SetIndexEmptyValue(1,0.0);
//---- clear buffers when reinitializing
   if(ArraySize(ExtMapBuffer1)>0) ArrayInitialize(ExtMapBuffer1,0.0);
   if(ArraySize(ExtMapBuffer2)>0) ArrayInitialize(ExtMapBuffer2,0.0);
//---- set labels for DataWindow
   if(ExtFormula==0)
     {
      SetIndexLabel(0,"Pivot");
      SetIndexLabel(1,NULL);
     }
    else
     {
      SetIndexLabel(0,"Resistance");
      SetIndexLabel(1,"Support");
     }
//----
   DeleteLines();
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
   DeleteLines();
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
  {
   int    counted_bars=IndicatorCounted();
   int    begin_bar, cnt;
   double hi=0, lo=0, cl;
   double hi1=0, lo1=0;
   double yesterday_high, yesterday_low, yesterday_close, today_open;
   double yesterday_high1, yesterday_low1;   
   double P, S, R, S05, R05, S10, R10, S15, R15, S20, R20, S25, R25, S30, R30;
//---- test parameters
   if(ExtFormula<0 || ExtFormula>3) return(-1);
   if(Period()>=PERIOD_D1)          return(-1);
   if(counted_bars==Bars-1)         return(-1);
//---- set check beginning
   begin_bar=Bars-counted_bars;
//----
   int    new_day=TimeDayOfWeek(Time[begin_bar]-ShiftTime);
   for(cnt=MathMax(begin_bar,1440/Period()); cnt>=0; cnt--)
     {
      if(TimeDayOfWeek(Time[cnt]-ShiftTime)==new_day)
        {
         if(High[cnt]>hi) hi=High[cnt];
         if(High[cnt+24]>hi1) hi1=High[cnt+24];
         if((Low[cnt]<lo)||(lo==0)) lo=Low[cnt];
         if((Low[cnt+24]<lo1)||(lo1==0)) lo1=Low[cnt+24];
         cl=Close[cnt];
        }
      else 
        {
         new_day=TimeDayOfWeek(Time[cnt]-ShiftTime);
         if(new_day>0 && new_day<6)
           {
            yesterday_close=cl;
            yesterday_high=hi;
            yesterday_low=lo;
            yesterday_high1=hi1;
            yesterday_low1=lo1;
            hi=0;lo=0;
            hi1=0;lo1=0;
            P = (yesterday_high + yesterday_low + yesterday_close) / 3;
           } 
         switch(ExtFormula)
           {
            case 1 :
               R = P + P - yesterday_low;
               S = P + P - yesterday_high;
               break;
            case 2 :
               R = P + yesterday_high - yesterday_low;
               S = P - yesterday_high + yesterday_low;
               break;
            case 3 :
               R = P + P - yesterday_low - yesterday_low + yesterday_high;
               S = P + P - yesterday_high - yesterday_high + yesterday_low;		
           }
        }  
       if(ExtFormula==0) ExtMapBuffer1[cnt]=P;
       else
         {
          ExtMapBuffer1[cnt]=R;
          ExtMapBuffer2[cnt]=S;
         }
     }
   P = NormalizeDouble((yesterday_high + yesterday_low + yesterday_close)/3,Digits);

   R10 = NormalizeDouble(yesterday_low,Digits);
   S10 = NormalizeDouble(yesterday_high,Digits);

   R05 = NormalizeDouble(yesterday_low1,Digits);
   S05 = NormalizeDouble(yesterday_high1,Digits);


   ObjectCreate("Pivot_Line", OBJ_HLINE, 0, 0, P);
   ObjectSet("Pivot_Line", OBJPROP_COLOR, Orange);
   ObjectSet("Pivot_Line", OBJPROP_STYLE, STYLE_SOLID);
   ObjectSetText("Pivot_Line","Pivot "+DoubleToStr(P,Digits));
   
   ObjectCreate("Pivot label", OBJ_TEXT, 0, Time[20], P);
   ObjectSetText("Pivot label", " Pivot", 8, "Arial", LimeGreen);   

   ObjectCreate("LOPS2_Line", OBJ_HLINE, 0, 0, R05);
   ObjectSet("LOPS2_Line", OBJPROP_COLOR, LOPS);
   ObjectSet("LOPS2_Line", OBJPROP_STYLE, STYLE_DOT);
   ObjectSetText("LOPS2_Line","LOPS2 "+DoubleToStr(R05,Digits));
   
   ObjectCreate("LOPS2 label", OBJ_TEXT, 0, Time[20], R05);
   ObjectSetText("LOPS2 label", " LOPS2", 8, "Arial", LOPS);


   ObjectCreate("LOPS1_Line", OBJ_HLINE, 0, 0, R10);
   ObjectSet("LOPS1_Line", OBJPROP_COLOR, LOPS);
   ObjectSet("LOPS1_Line", OBJPROP_STYLE, STYLE_DOT);
   ObjectSetText("LOPS1_Line","LOPS1 "+DoubleToStr(R10,Digits));
   
   ObjectCreate("LOPS1 label", OBJ_TEXT, 0, Time[20], R10);
   ObjectSetText("LOPS1 label", " LOPS1", 8, "Arial", LOPS);

 
   ObjectCreate("HOPS2_Line", OBJ_HLINE, 0, 0, S05);
   ObjectSet("HOPS2_Line", OBJPROP_COLOR, HOPS);
   ObjectSet("HOPS2_Line", OBJPROP_STYLE, STYLE_DOT);
   ObjectSetText("HOPS2_Line","S0.5 "+DoubleToStr(S05,Digits));
   
   ObjectCreate("HOPS2 label", OBJ_TEXT, 0, Time[20], S05);
   ObjectSetText("HOPS2 label", " HOPS2", 8, "Arial", HOPS);

   ObjectCreate("HOPS1_Line", OBJ_HLINE, 0, 0, S10);
   ObjectSet("HOPS1_Line", OBJPROP_COLOR, HOPS);
   ObjectSet("HOPS1_Line", OBJPROP_STYLE, STYLE_DOT);
   ObjectSetText("HOPS1_Line","HOPS1 "+DoubleToStr(S10,Digits));
   
   ObjectCreate("HOPS1 label", OBJ_TEXT, 0, Time[20], S10);
   ObjectSetText("HOPS1 label", " HOPS1", 8, "Arial", HOPS);


//---- force objects drawing
   ObjectsRedraw();
//----
   return(0);
  }
//+------------------------------------------------------------------+